{Copyright (C) 2006  Benito van der Zander

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}
{
How to use it:
  1. create a new object with this options:
       version:
         Integer based incrementing version number
         for example 1, next version, 2.
         You cann't use 1.6, or 1.56, and 156>16
       installDir:
         Directory to the program files to be updated
       versionURL:
         URL containing a file containing only one number, the
         currently newest version of the program. (don't even start a
         new line at the end)
       versionDetailsURL:
         URL containing an ini file with details to the newest version.
         In the current version of TAutoUpdater this file contains only
         a section [version-number] with a string package-url containing 
         the ZIP-archive to download. (there can be more, but it has no use)
  2. call existsUpdate to compare the current version number with the one 
     in the web
     YOU NEED ACCESS TO VERSIONURL (= INTERNET).
  3. call hasDirectoryWriteAccess to check, if you are allowed to change the
     files in the program directory.
     If not, you have to restart the whole process with changed rights (->admin)
  4. call downloadUpdate(tempDir) to download the file to tempDir.
     tempDir can be omitted.
  5. call installUpdate to extract the zip archive and copy it to the program path
     YOU NEED ACCESS TO THE PROGRAMPATH
  6. call needRestart to check if the program must be restarted.
     Changes to the executable itself will only be installed after restart. 
     (using a batch file)

  benito@benibela.de
}

unit autoupdate;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils,internetaccess;
type

{ TAutoUpdater }

TAutoUpdater=class
private
  finternet:TInternetAccess;
  fupdateExisting,fneedBAT:boolean;
  fcurrentVersion,fnewversion:integer;
  finstallDir,fversionURL,fversionDetailsURL: string;
  ftempDir:string;
public
  constructor create(currentVersion:integer;installDir,versionURL,versionDetailsURL: string);
  function hasDirectoryWriteAccess:boolean;
  function existsUpdate:boolean;
  (** downloads the update to $tempDir\$newVersion
  *)
  procedure downloadUpdate(tempDir:string='');
  (** can every old file be replaced
     Result:
       true: yes, call installUpdate
       false: no, call installUpdate and RESTART (you must restart)
  *)
  function isCurrentFileUpdated:boolean;
  (** overwrites files and call additional rewrite process *)
  procedure installUpdate;
  destructor destroy;override;
  
  property newVersion: integer read fnewversion;
  property needRestart: boolean read fneedBat;
end;
implementation
uses w32internetaccess,inifiles,unzip,ziptypes,windows;

{ TAutoUpdater }

function TAutoUpdater.hasDirectoryWriteAccess: boolean;
var f:THandle;
begin
  if Win32Platform=VER_PLATFORM_WIN32_WINDOWS then result:=true
  else if Win32Platform=VER_PLATFORM_WIN32_NT then begin
    f:=CreateFile(pchar(copy(finstallDir,1,length(finstallDir)-1) ),GENERIC_WRITE, FILE_SHARE_WRITE or FILE_SHARE_READ, nil,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL or FILE_FLAG_BACKUP_SEMANTICS,0);
    if f=INVALID_HANDLE_VALUE then result:=false
    else begin
      result:=true;
      closehandle(f);
    end;
  end else result:=false;
end;

constructor TAutoUpdater.create(currentVersion: integer; installDir,
  versionURL, versionDetailsURL: string);
begin
  fcurrentVersion:=currentVersion;
  finstallDir:=installDir;
  if finstallDir[length(finstallDir)]<>'\' then
    finstallDir:=ExtractFilePath(ParamStr(0));
  fversionURL:=versionURL;
  fversionDetailsURL:=versionDetailsURL;
end;

function TAutoUpdater.existsUpdate: boolean;
  function deleteNewLine(s:string):string;
  begin
    if s[length(s)] in [#13,#10] then
      result:=copy(s,1,length(s)-2)
     else
      result:=s;
  end;
begin
  if finternet=nil then
    finternet:=TW32InternetAccess.create; //TODO: generic internet
  result:=true;
  try
    fnewversion:=StrToInt(deleteNewLine(finternet.get(fversionURL)));
    result:=fnewversion>fcurrentVersion;
  except
    result:=false;
  end;
end;

function sayYes( Rec : pReportRec ) : Boolean;
begin
  result:=true;
end;

procedure TAutoUpdater.downloadUpdate(tempDir: string);
var ini:TIniFile;
    sl:TStringList;
    //str:TMemoryStream;
    pack:string;
    f:file;//TFileStream;
begin
  ftempDir:=tempDir;
  if ftempDir='' then
    ftempDir:=GetTempDir();
  if ftempDir[length(ftempDir)]<>'\' then
    ftempDir:=ftempDir+'\';
  ftempDir:=ftempDir+IntToStr(fnewversion)+'\';
  
  try
    RemoveDir(copy(ftempDir,1,length(ftempdir)-1));
    mkdir(copy(ftempDir,1,length(ftempdir)-1));
  except
    if not DirectoryExists(copy(ftempDir,1,length(ftempdir)-1)) then
      raise exception.create('Temporres Verzeichnis '+ftempDir+' konnte nicht erstellt werden');
  end;
  //str:=TMemoryStream.create;
  sl:=TStringList.create;
  sl.text:=finternet.get(fversionDetailsURL);
  sl.SaveToFile(ftempDir+'_#_UPDATE_DETAILS_#_.ini');
  ini:=TIniFile.Create(ftempDir+'_#_UPDATE_DETAILS_#_.ini');
//  ini.SetStrings(sl);
  pack:=ini.ReadString(IntToStr(fnewversion),'package-url','');
  if pack='' then raise Exception.Create('Detaillierte Updateinformationen sind nicht verfgbar');
  ini.free;
//  str.free;
  sl.free;
  DeleteFile(pchar(ftempDir+'_#_UPDATE_DETAILS_#_.ini'));


  pack:=finternet.get(pack);
//  f:=TFileStream.Create(ftempDir+'_#_UPDATE_#_UPDATE_#_.zip',fmCreate);
  //f.Write(pack,length(pack));
//  f.Free;
  assign(f,ftempDir+'_#_UPDATE_#_UPDATE_#_.zip');
  rewrite(f,1);
  BlockWrite(f,pack[1],length(pack));
  close(f);

  if FileUnzip(pchar(ftempDir+'_#_UPDATE_#_UPDATE_#_.zip'),pchar(ftempDir),'*.*',nil,@sayYes)=0 then
    raise Exception.Create('Update konnte nicht entpackt werden');

  DeleteFile(pchar(ftempDir+'_#_UPDATE_#_UPDATE_#_.zip'));

  fupdateExisting:=true;
  //result:=fupdateExisting;
  fneedBAT:=false;
end;

function TAutoUpdater.isCurrentFileUpdated: boolean;
begin
  result:=not FileExists(ftempDir+ExtractFileName(ParamStr(0)));
  fneedBAT:=result;
end;

procedure TAutoUpdater.installUpdate;
var frec:TSearchRec;
    myName:string;
    sl:TStringList;
    deltree:string;
begin
  myName:=lowercase(ExtractFileName(ParamStr(0)));
  FindFirst(ftempDir+'*.*',faAnyFile,frec);
  repeat
    if (frec.Name='') or (frec.Name[1]='.')  then continue;
    if lowercase(frec.Name)=myName then begin
      fneedbat:=true;
      continue;
    end;
    //copyFile(ftempDir+frec.Name,finstallDir+frec.Name);
    //DeleteFile(ftempDir+frec.Name)
    RenameFile(ftempDir+frec.Name,finstallDir+frec.Name);//should move it
    DeleteFile(pchar(ftempDir+frec.Name));
  until FindNext(frec)<>0;
  sysutils.FindClose(frec);
  
  if Win32Platform=VER_PLATFORM_WIN32_NT then deltree:='rd /S /Q'
  else deltree:='deltree /Y';

  
  if fneedBAT then begin
    sl:=TStringList.Create;
    sl.text:='@echo off'#13#10+
             'echo Update wird installiert...'+#13#10+
             ':Marke'#13#10+
             'Del "'+finstallDir+myName+'"'#13#10+
             'If EXIST "'+finstallDir+myName+'" Goto Marke'#13#10+
             'copy "'+ftempDir+myName+'" "'+finstallDir+myName+'"'#13#10+
//             'del "'+ftempDir+myName+'"'#13#10+
             '"'+finstallDir+myName+'" /updated-to '+inttostr(fnewversion)+#13#10+
             deltree+' "'+copy(ftempDir,1,length(ftempdir)-1)+'"'+#13#10
             ;
//             'del "'+ftempDir+'replaceExe.bat"'#13#10+
    sl.SaveToFile(ftempDir+'replaceExe.bat');
    sl.free;
    winexec(pchar('"'+ftempDir+'replaceExe.bat"'),sw_hide);
  end else RemoveDirectory(pchar(copy(ftempDir,1,length(ftempdir)-1)));
end;

destructor TAutoUpdater.destroy;
begin
  if finternet=nil then
    finternet.free;
  inherited destroy;
end;

end.

